#!/usr/bin/env python

"""
Directory output context.

Copyright (C) 2018, 2019 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.output.common import Output
from moinformat.utils.directory import Directory
from os.path import extsep, join, sep

class DirectoryOutput(Output):

    "A directory output context."

    name = "directory"

    def __init__(self, metadata):

        "Initialise the context with the given 'metadata'."

        if not metadata.has_key("output_filename"):
            raise ValueError, metadata

        Output.__init__(self, metadata)
        self.dir = Directory(metadata.get("output_filename"))
        self.dir.ensure()

        # Support an encoding of the level separator for the filesystem.
        # Where it is the same as the directory separator, documents are stored
        # using nested directories, not as a flat list.

        self.level_sep = metadata.get("output_separator", sep)

        # Use any document index setting as the default for the index filename.

        document_index = metadata.get("document_index", "index.html")
        self.index_name = metadata.get("index_name", document_index)
        self.root_pagename = metadata.get("root_pagename")
        self.attachments_dir = metadata.get("attachments")

        # Support a common attachments directory.

        self.common_attachments = metadata.get("common_attachments")

    def _get_attachments_dir(self, pagename):

        "Return the attachments directory for 'pagename'."

        if self.common_attachments:
            return join(self.dir.filename, self.attachments_dir)
        else:
            return join(self.dir.filename, self.attachments_dir, pagename)

    # Convenience methods.

    def ensure(self, pagename):

        "Ensure that the given 'pagename' exists."

        if not pagename:
            return None

        self.dir.ensure(self.to_filename(pagename))

    def ensure_attachments(self, pagename):

        "Ensure that attachment storage for the given 'pagename' exists."

        if not pagename:
            return None

        self.dir.ensure(self._get_attachments_dir(pagename))

    def get_attachment_filename(self, pagename, filename):

        """
        Return the full path of an attachment file for the given 'pagename'
        having the given 'filename'.
        """

        if not pagename:
            return None

        return self.dir.get_filename(join(self._get_attachments_dir(pagename), filename))

    def get_filename(self, filename):

        """
        Return the full path of a file with the given 'filename' found within
        the directory. The full path is an absolute path.
        """

        return self.dir.get_filename(filename)

    # Name translation methods.

    def to_filename(self, pagename):

        "Return the filename corresponding to 'pagename'."

        # Encode hierarchical filenames.

        if self.level_sep == sep:

            # For the root page, use the top-level directory.

            if pagename == self.root_pagename:
                return ""
            else:
                return pagename

        # Encode single-directory filenames.

        else:
            return self.level_sep.join(pagename.split("/"))

    def to_pagename(self, filename):

        "Return the pagename corresponding to 'filename'."

        # Encode pagenames from hierarchical filenames.

        if self.level_sep == sep:
            return self.within(filename)

        # Encode pagenames from single-directory filenames.

        else:
            return "/".join(filename.split(self.level_sep))

    # Serialisation methods.

    def can_write(self):

        "Return whether this context supports page writing."

        return True

    def writefile(self, text, filename, encoding=None):

        """
        Write 'text' to the file having the given 'filename'. If the
        optional 'encoding' is specified, override the general encoding.
        """

        return self.writepath(text, self.dir.get_filename(filename), encoding)

    def writepage(self, text, pagename, encoding=None):

        """
        Write 'text' to the file having the given 'pagename' and optional
        'encoding'. If 'parent' is specified and a true value, it indicates that
        the page is a parent of other pages.
        """

        filename = self.to_filename(pagename)

        # For hierarchical storage, store the page inside a directory bearing
        # its name.

        if self.level_sep == sep:

            # Make a directory for the page.

            if not self.dir.exists(filename):
                self.dir.makedirs(filename)

            # Write to an index filename within any existing directory.

            filename = join(filename, self.index_name)

        self.writefile(text, filename, encoding)

output = DirectoryOutput

# vim: tabstop=4 expandtab shiftwidth=4
