#!/usr/bin/env python

"""
Moin wiki table parser.

Copyright (C) 2017, 2018, 2021 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.parsers.common import get_patterns, excl, group
from moinformat.parsers.moin import MoinParser
from moinformat.serialisers import serialise
from moinformat.tree.moin import Break, Table, TableAttrs, TableCell, \
                                 TableRow, Text
from moinformat.tree.table import Continuation

join = "".join

# Parser functionality.

class TableParser(MoinParser):

    "A parser for improved table syntax."

    formats = ["table"]

    # Principal parser methods.

    def parse_region_content(self, items, region):

        """
        Parse the data provided by 'items' to populate the given 'region'. For
        table regions, normal region handling is wrapped by management of the
        table structure.
        """

        self.set_region(items, region)

        # Start to populate table rows.

        cell = TableCell([])
        row = TableRow([cell])
        table = Table([row])
        self.append_node(self.region, table)
        self.new_block(cell)

        while True:
            self.parse_region_details(cell, self.region_pattern_names)

            # Suppress any initial paragraph break within a table cell. This is
            # caused by a blank line which might be employed for clarity.

            first = cell.node(0)

            if isinstance(first, Break):
                cell.remove(first)
                cell.padding += serialise(first, self.get_serialiser())

            # Detect the end of the table.

            pattern = self.matching_pattern()

            if not pattern or pattern == "regionend":
                break

            elif pattern == "columnsep":
                cell = TableCell([], leading=self.match_group("leading"),
                                     padding=self.match_group("padding"))
                row.append(cell)
                self.new_block(cell)

            elif pattern == "rowsep":
                row = TableRow([], leading=self.match_group("leading"),
                                   padding=self.match_group("padding"))
                table.append(row)
                cell = TableCell([])
                row.append(cell)
                self.new_block(cell)

    # Parser handler methods.

    def parse_continuation(self, cell):

        "Handle continuation padding."

        feature = self.match_group("feature")
        self.append_inline(cell, Continuation(feature))

    def parse_table_attrs(self, cell):

        "Handle the start of table attributes within 'cell'."

        MoinParser.parse_table_attrs(self, cell)

        if isinstance(cell.node(-1), TableAttrs):
            self.new_block(cell)

    # Regular expressions.

    syntax = {}
    syntax.update(MoinParser.syntax)
    syntax.update({
        # At start of line:

        "rowsep"        : join((group("leading", r"\s*"),       # ws... (optional)
                                "^==",                          # ==
                                excl(r".*==\s*?$"),             # not-heading
                                group("padding", r"\N*\n?"))),  # ws-excl-nl... nl (optional)

        "continuation"  : group("feature",
                          join((group("leading", r"\s*"),       # ws... (optional)
                                "^",
                                group("indent", r"\s*"),        # ws... (optional)
                                r"\.\.",                        # ..
                                excl(r"\."),                    # not-.
                                group("padding", r"\N*")))),    # ws-excl-nl... (optional)

        # Within text:

        "columnsep"     : join((group("leading", r"\s*"),       # ws... (optional)
                                r"\|\|",                        # ||
                                excl(r"\|"),                    # not-|
                                group("padding", r"\N*\n?"))),  # ws-excl-nl... nl (optional)
        })

    patterns = get_patterns(syntax)



    # Pattern details.

    region_pattern_names = [
        "columnsep", "continuation", "rowsep", "tableattrs",
        ] + MoinParser.region_without_table_pattern_names



    # Pattern handlers.

    end_region = MoinParser.end_region
    parse_table_end = MoinParser.parse_region_end

    handlers = {}
    handlers.update(MoinParser.handlers)
    handlers.update({
        "columnsep" : end_region,
        "continuation" : parse_continuation,
        "rowsep" : end_region,
        "regionend" : parse_table_end,
        "tableattrs" : parse_table_attrs,
        })

parser = TableParser

# vim: tabstop=4 expandtab shiftwidth=4
